#-------------------------------------------------------------------------------
# Contact: Dave Culyba
# Created: Jan 2009
#
# Purpose: Used to render data view to image space
#-------------------------------------------------------------------------------

from core import util
#import basic pygame modules
import pygame
from pygame.locals import *

class Camera:
	
	def __init__(self, daisyChain, width, height):
		self.daisyChain = daisyChain
		self.width = width
		self.height = height
		self.centerY = self.daisyChain.screenRect.centery
		
		self.ZOOMED_IN_CENTERY = self.daisyChain.screenRect.centery - self.daisyChain.screenRect.height/6
		self.ZOOMED_IN_HEIGHT = self.daisyChain.screenRect.height/3
		
		self.ZOOMED_OUT_CENTERY = self.daisyChain.screenRect.centery
		self.ZOOMED_OUT_HEIGHT = self.height

		self.height = util.interpolate(self.ZOOMED_IN_HEIGHT, self.ZOOMED_OUT_HEIGHT, 0.0)
		self.centerY = util.interpolate(self.ZOOMED_IN_CENTERY, self.ZOOMED_OUT_CENTERY, 0.0)
		
	def getCameraHeightOffset(self):
		return -(self.centerY - self.daisyChain.screenRect.centery)

	def getTopRect(self):
		topAmount = self.daisyChain.screenRect.centery - self.height/2
		if (topAmount < 0):
			topAmount = 0
		topRect = pygame.rect.Rect(self.daisyChain.screenRect.left, self.daisyChain.screenRect.top, self.daisyChain.screenRect.width, topAmount)
		return topRect
	
	def getBottomRect(self):
		bottomOfVisible = self.daisyChain.screenRect.centery + self.height/2
		if (bottomOfVisible < self.daisyChain.screenRect.bottom):
			bottomAmount = self.daisyChain.screenRect.bottom - bottomOfVisible
		else:
			bottomAmount = 0
		bottomRect = pygame.rect.Rect(self.daisyChain.screenRect.left, self.daisyChain.screenRect.bottom - bottomAmount, self.daisyChain.screenRect.width, bottomAmount+1)
		return bottomRect
	
	def drawLetterBoxes(self):
		pygame.draw.rect( self.daisyChain.screen, pygame.color.THECOLORS["black"], self.getTopRect() )
		pygame.draw.rect( self.daisyChain.screen, pygame.color.THECOLORS["black"], self.getBottomRect() )
	
	def setPercent(self, percent):
		self.height = util.interpolate(self.ZOOMED_IN_HEIGHT, self.ZOOMED_OUT_HEIGHT, percent)
		self.centerY = util.interpolate(self.ZOOMED_IN_CENTERY, self.ZOOMED_OUT_CENTERY, percent)

