#-------------------------------------------------------------------------------
# Contact: Dave Culyba
# Created: Jan 2009
#
# Purpose: Logs the events of the game to be saved and read back
#-------------------------------------------------------------------------------

class EventLog:
	
	#define the message types
	KEY_DOWN = "KEY_DOWN"
	KEY_UP = "KEY_UP"
	FLOWER_GREW = "FLOWER_GREW"
	FLOWER_PULLED = "FLOWER_PULLED"
	
	eventTypes = (KEY_DOWN, KEY_UP, FLOWER_GREW, FLOWER_PULLED)
	
	failedLoad = False
	
	def __init__(self, name = "No Name"):
		self.playerName = name
		self.events = []
		
	def addEvent(self, tickCount, eventName, eventData):
		self.events.append((tickCount, eventName, eventData))
	
	def checkType(self, eventToCheck):
		for eventType in self.eventTypes:
			if (eventType == eventToCheck):
				return True
		return False
	
	def initFromData(self, data):
		print "loading..."
		self.failedLoad = False
		if (type(data) <> list or len(data) <> 2):
			print "BAD DATA", data, type(data)
			self.failedLoad = True
		if (not self.failedLoad):
			if (type(data[0]) == str or type(data[0]) == unicode):
				self.playerName = data[0]
			else:
				print "BAD NAME DATA", data[0], type(data[0])
				self.failedLoad = True
		if (not self.failedLoad):
			for datum in data[1]:
				if (len(datum) <> 3):
					print "SUB DATA IS WRONG FORMAT. LENGTH IS", len(data)
					self.failedLoad = True
					break
				else:
					if (type(datum[0]) <> int):
						print "TICK COUNT TYPE IS BAD", datum[0]
						self.failedLoad = True
						break;
					tickCount = datum[0]
					
					eventName = datum[1]
					if (not self.checkType(eventName)):
						print "EVENT TYPE IS BAD", eventName
						failed = True
						break;
					
					if (type(datum[2]) <> list):
						print "DATA TYPE IS BAD", datum[2]
						self.failedLoad = True
						break;
					eventData = datum[2]
					print eventData
					self.addEvent(tickCount, eventName, eventData)
		if (self.failedLoad):
			self.playerName = "FAILED LOAD: NO NAME"
			self.events = []
	
	def getData(self):
		return (self.playerName, self.events)
	
	def printEvents(self):
		print "events for ", this.playerName
		for event in self.events:
			print event
			
	def getTopEvent(self):
		if (len(self.events) > 0):
			return self.events[0]
		else:
			return None
	
	def getTopEventTickTime(self):
		if (len(self.events) > 0):
			return self.events[0][0]
		else:
			return -1
	
	def removeTopEvent(self):
		self.events = self.events[1:len(self.events)]
		
	def writeLog(self, outputFile):
		try:
			f = open(outputFile, 'w')
			f.write(self.playerName+"\n")
			for event in self.events:
				eventString = str(event[0])+" "+str(event[1])+" [" + str(event[2][0])
				for data in event[2][1:]:
					eventString += ","+str(data)
				eventString += "]\n"
				f.write(eventString)
			f.close()		
		except IOError:
			print "Failed to write to outputFile"

	def readLog(self, inputFile):
		try:
			f = open(inputFile, 'r')
			self.playerName = f.readline().strip()
			self.failedLoad = False
			for line in f:
				args = line.strip().split(' ')
				#if the number of arguments 
				if (len(args) <> 3):
					self.failedLoad = True
					break
				#get the tick count
				if (not(args[0].isdigit())):
					print "TICK FAIL", args[0]
					self.failedLoad = True
					break
			 	tickTime = int(args[0])
				#get the type 
				typeMatches = self.checkType(args[1])
				if (not(typeMatches)):
					print "TYPE FAIL", args[1]
					self.failedLoad = True
					break
				eventType = args[1]
				#get the data
				data = args[2].strip('[]').split(',')
				eventData = []
				for datum in data:
#					if (not(datum.isdigit())):
#						print "DATA FAIL", datum
#						self.failedLoad = True
#						break
			 		eventData.append(int(datum))
			 	
			 	self.addEvent(tickTime, eventType, eventData)
		except IOError:
			self.failedLoad = True
		if (self.failedLoad):
			print "FAILED TO READ INPUT LOG"
		 	self.events = []
		 
		f.close()
		 
		 