#------------------------------------------------------------------
# Contact: Jason Pratt
# Created: Jan 2009
#------------------------------------------------------------------

import os
import time

import pygame

import util

class FarewellScreen:
	def __init__( self ):
		self.credits = util.load_image( os.path.abspath( os.path.join( "assets", "textures", "theend.png" ) ) )
		self.startTime = 0.0
		self.numPicked = 10
		self.numKilled = 8
		self.summaryMessageTime = 18.0  # in seconds
		self.mode = "summaryMessage"
		self.summaryMessageLines = ["In the land of stick people and daisies,",
                                    "some choose to pick flowers, spread pollen,",
									"and share the joy, while others choose to",
									"rip flowers by their roots, killing life",
									"and happiness at the same time.",
									"",
									"What choices have you made, and how might",
									"those choices affect those who come after?",]

		self.summaryMessageColor = (0, 0, 0)
		self.summaryMessagePos = (40, 80)
		self.summaryMessageLineSpacing = 42
		self.summaryMessageTextHeight = 34
		self.summaryMessageFont = pygame.font.Font( "freesansbold.ttf", self.summaryMessageTextHeight )

		self.resultsColor = (255, 255, 255)
		self.resultsPos = (25, 500)
		self.resultsLineSpacing = 45
		self.resultsTextHeight = 43
		self.resultsFont = pygame.font.Font( "freesansbold.ttf", self.resultsTextHeight )
		self.pickedResults = "You picked %d daisies, making %d new seeds."
		self.killedResults = "You ripped %d daisies from their roots."

	def startFarewell( self, numPicked, numKilled ):
		self.startTime = time.time()
		self.numPicked = numPicked
		self.numKilled = numKilled

	def update( self ):
		if time.time() - self.startTime > self.summaryMessageTime:
			self.mode = "credits"

	def render( self, surface ):
		if self.mode == "summaryMessage":
			surface.fill( (255, 255, 255) )
			for i in range( len( self.summaryMessageLines ) ):
				text = self.summaryMessageLines[i]
				pos = (self.summaryMessagePos[0], self.summaryMessagePos[1] + i*self.summaryMessageLineSpacing)
				summaryLine = self.summaryMessageFont.render( text, True, self.summaryMessageColor )
				surface.blit( summaryLine, pos )
		else:
			surface.blit( self.credits, (0, 0) )
			pickedText = self.pickedResults % (self.numPicked, self.numPicked*2)
			pickedLine = self.summaryMessageFont.render( pickedText, True, self.resultsColor )
			surface.blit( pickedLine, self.resultsPos )
			killedText = self.killedResults % self.numKilled
			killedLine = self.summaryMessageFont.render( killedText, True, self.resultsColor )
			surface.blit( killedLine, (self.resultsPos[0], self.resultsPos[1] + self.resultsLineSpacing) )


