#------------------------------------------------------------------
# Contact: Shawn Patton
# Created: Jan 2009
#
# Purpose: The flower a player picks
#
#------------------------------------------------------------------

# Python imports
import pygame
from pygame.locals import *

import math
import random

# Daisy Chain imports
from core import util
# from core import Ground
from core.Animator import Animator

class Flower(pygame.sprite.Sprite):
	# flower states
	SEED = 0
	SEEDLING = 1
	BUD = 2
	BLOOM = 3
	STALK = 4
	DEAD = 5
	
	# internal counter to ensure unique ids
	flowerCount = 0
	
	def __init__( self, daisyChain, groups, ground, x, startState):
		self.daisyChain = daisyChain
		self.ground = ground
		self.id = self.getUniqueId()
		self.x = x
		self.y = 0
		self.state = startState
		self.frame = 0 # frame of the animation of current bloom
		self.oldState = ""
		self.curAnimator = None
		self.seedAnimator = None
		self.seedlingAnimator = None
		self.budAnimator = None
		self.bloomAnimator = None
		self.stalkAnimator = None
		self.deadAnimator  = None
		self.value = self.daisyChain.flowerBaseValue
		self.stateToAnimatorInfo = { # state:  (animator, pathPrefix, # frames)
								Flower.SEED : [self.seedAnimator, 'flower/seed/goldenSeed/seed_%d.png', 3, 6],
								Flower.SEEDLING : [self.seedlingAnimator, 'flower/seedling/seedling_%d.png', 7, 6],
								Flower.BUD : [self.budAnimator, 'flower/bud/bud_%d.png', 1, 6],
								Flower.BLOOM : [self.bloomAnimator,'flower/fullBloom/fullBloom_%d.png', 1, 6],
								Flower.STALK : [self.stalkAnimator, 'flower/stalk/stalk_%d.png', 1, 6],
								Flower.DEAD : [self.deadAnimator, 'flower/dead/dead_%d.png', 1, 6],		
		}
		self.initAnimators()
		# drawing stuff
		pygame.sprite.Sprite.__init__( self, groups )
		self.initImages()
		self.frame = 0
		
	def initAnimators( self, ):
		self.seedAnimator = Animator(self, self.stateToAnimatorInfo[Flower.SEED][1], self.stateToAnimatorInfo[Flower.SEED][2], self.stateToAnimatorInfo[Flower.SEED][3] )
		self.seedlingAnimator = Animator(self, self.stateToAnimatorInfo[Flower.SEEDLING][1], self.stateToAnimatorInfo[Flower.SEEDLING][2], self.stateToAnimatorInfo[Flower.SEEDLING][3] )
		self.budAnimator = Animator(self, self.stateToAnimatorInfo[Flower.BUD][1], self.stateToAnimatorInfo[Flower.BUD][2], self.stateToAnimatorInfo[Flower.BUD][3] ) 
		self.bloomAnimator = Animator(self, self.stateToAnimatorInfo[Flower.BLOOM][1], self.stateToAnimatorInfo[Flower.BLOOM][2], self.stateToAnimatorInfo[Flower.BLOOM][3] )
		self.stalkAnimator = Animator(self, self.stateToAnimatorInfo[Flower.STALK][1], self.stateToAnimatorInfo[Flower.STALK][2], self.stateToAnimatorInfo[Flower.STALK][3] )
		self.deadAnimator = Animator(self, self.stateToAnimatorInfo[Flower.DEAD][1], self.stateToAnimatorInfo[Flower.DEAD][2], self.stateToAnimatorInfo[Flower.DEAD][3] )
		

	def getUniqueId( self ):
		Flower.flowerCount += 1
		return Flower.flowerCount
		
	def setState( self, newState ):
		if newState == self.state:
			print "WARNING: Flower.setState( newState=%s ) when current state is the same. Ignoring setState call." % newState
			return
		self.oldState = self.state
		self.state = newState
		
	def isPickable( self ):
		if self.state in (Flower.BUD, Flower.BLOOM):
			return True
		else:
			False
			
	def isPicked( self ):
		return (self.state == Flower.STALK)
	
	def isPullable( self ):
		if self.state in (Flower.BUD, Flower.BLOOM, Flower.STALK):
			return True
		else:
			return False
			
	def initImages( self ):
		self.image = self.seedlingAnimator.images[0]
		self.rect = self.image.get_rect(midbottom=self.daisyChain.screenRect.midbottom)
	
	def pick( self ):
		self.setState( Flower.STALK )
		# launch 2 pollen piece upward
		self.ground.releasePollen(self.x)
	
	def pollinate( self ):
		if self.state == Flower.BUD:
			self.setState(Flower.BLOOM)
		self.value += self.daisyChain.pollinateBonus
	
	def kill( self, args ):
		# die
		for group in self.groups():
			group.remove(self)
		self.ground.removeFlower(self)
		
		
	def simulateStep( self, dT ):
		# check state- has it changed?
		
		if self.oldState != self.state:
			if self.curAnimator != None:
				self.curAnimator.reset()
				self.oldState = self.state
				
		if self.state == self.SEED:
			self.curAnimator = self.seedAnimator
			self.y = 30
			self.curAnimator.animate(dT, self.setState, [self.SEEDLING])
		elif self.state == self.SEEDLING:
			self.curAnimator = self.seedlingAnimator
			self.curAnimator.animate(dT, self.setState, [self.BUD])
			self.y = -25
		elif self.state == self.BUD:
			self.curAnimator = self.budAnimator
			self.curAnimator.animate(dT, self.setState, [self.BLOOM])
			self.y = -80
		elif self.state == self.BLOOM:
			self.curAnimator = self.bloomAnimator
			self.curAnimator.animate(dT)
			self.y = -80
		elif self.state == self.STALK:
			self.curAnimator = self.stalkAnimator
			self.curAnimator.animate(dT, self.kill, [None])
			self.y = -80
		elif self.state == self.DEAD:
			self.curAnimator = self.deadAnimator
			self.curAnimator.animate(dT, self.kill, [None])
			self.y = -80
			
	def update( self ):
		self.rect.centerx = self.x
		self.rect.centery = self.ground.getImageSpaceHeight()+ self.y
		
