#-------------------------------------------------------------------------------
# Contact: Dave Culyba
# Created: Jan 2009
#
# Purpose: Ground layer that handles flowers in it.
#-------------------------------------------------------------------------------

from core import util
#import basic pygame modules
import pygame

from pygame.locals import *

class Particle(pygame.sprite.Sprite):

	FLOAT_FORCE = -6
	FRICTION_AMOUNT = .5

	def __init__(self, daisyChain, groups, player, imageFile, spawnPoint, initialForce, lifetime, startTime):
		pygame.sprite.Sprite.__init__(self, groups)
		self.image = util.load_image('assets/textures/' + imageFile ).convert_alpha()
		self.player = player
		self.daisyChain = daisyChain
		self.x = spawnPoint[0]
		self.y = spawnPoint[1]
		self.lifetime = lifetime
		self.totalTime = 0
		self.startTime = startTime
		
		self.velocityX = initialForce[0]
		self.velocityY = initialForce[1]
		
		self.originalImage = self.image
		self.rect = self.image.get_rect(center=spawnPoint)
		if (startTime != 0):
			self.image = util.get_alphaed(self.originalImage, 0)
		
	def isDone(self):
		return (self.totalTime + self.startTime > self.lifetime)
	
	def hasStarted(self):
		return self.totalTime >= self.startTime

	def update(self):
		self.rect.centerx = self.x
		self.rect.centery = self.y + self.daisyChain.camera.getCameraHeightOffset()
		
	def simulateStep( self, dtInSeconds ):
		if (not self.isDone()):
			self.totalTime += dtInSeconds
			if (self.hasStarted()):
				if (self.isDone()):
					for group in self.groups():
						group.remove(self)
				else:
					self.velocityX += (0 - self.velocityX)*self.FRICTION_AMOUNT*dtInSeconds
					self.velocityY += self.FLOAT_FORCE*dtInSeconds
					
					self.x += self.velocityX*dtInSeconds
					self.y += self.velocityY*dtInSeconds
					
					startAlphaTime = self.lifetime/3
					timeElapsed = self.totalTime - self.startTime
					if (timeElapsed < startAlphaTime):
						alphaValue = 255
					else:
						alphaValue = (1 - (timeElapsed - startAlphaTime) / (self.lifetime - startAlphaTime)) * 255
					self.image = util.get_alphaed(self.originalImage, alphaValue)#alphaValue
					#self.animator.animate(dtInSeconds)
