#------------------------------------------------------------------
# Contact: Shawn Patton
# Created: Jan 2009
#
# Purpose: The pollen that floats up and grows
#
#------------------------------------------------------------------
import pygame
from pygame.locals import *


# Python imports
import pygame
from pygame.locals import *

import math
# Daisy Chain imports
from core import util
from core.Animator import Animator
from core.Flower import Flower

class Pollen(pygame.sprite.Sprite):
	
	# internal counter to ensure unique ids
	pollenCount = 0
	SWIRL = 0
	FLOAT = 1
	
	
	def __init__( self, daisyChain, groups, ground, ceiling, x, destX, period):		
		pygame.sprite.Sprite.__init__( self, groups )
		self.daisyChain = daisyChain
		self.xBias = 0
		self.ground = ground
		self.ceiling = ceiling
		self.id = self.getUniqueId()
		self.x = x
		self.initialX = self.x
		self.destX = destX
		self.y = ground.dataTop
		self.initialY = self.y
		
		self.state = self.SWIRL
		self.oldState = None
		self.period = period
		self.initAnimators()
		# drawing stuff
		self.initImages()
		self.curAnimator = None
		self.yOffset = 0
		self.yOffsetTarget = self.rect.height/2
		
	def initAnimators( self, ):
		self.swirlAnimator = Animator(self, 'flower/pollen/pollen_%d.png', 8, 12) 
		self.floatAnimator = Animator(self, 'flower/seed/goldenSeed/seed_%d.png', 3, 6) 	

	def setState( self, newState ):
		if newState == self.state:
			print "WARNING: Pollen.setState( newState=%s ) when current state is the same. Ignoring setState call." % newState
			return
		self.oldState = self.state
		self.state = newState
		
	def getUniqueId( self ):
		Pollen.pollenCount += 1
		return Pollen.pollenCount
		
	def initImages( self ):
		self.image = self.swirlAnimator.images[0]
		self.rect = self.image.get_rect(midbottom=self.daisyChain.screenRect.midbottom)
	
	def kill( self ):
		# die
		for group in self.groups():
			group.remove(self)
		self.ground.removePollen(self)
		self.ceiling.growFlower(self.destX, Flower.SEED)
		
		
	def simulateStep( self, dT ):
		# check state- has it changed?
		# to make this "float upward", we decrease your y each time
		
		if self.oldState != self.state:
			if self.curAnimator != None:
				self.curAnimator.reset()
				self.oldState = self.state
				
		if self.state == self.SWIRL:
			self.curAnimator = self.swirlAnimator			
			self.curAnimator.animate(dT, self.setState, [self.FLOAT])
			
		elif self.state == self.FLOAT:
			self.curAnimator = self.floatAnimator			
				
		self.y = self.y - 1
		
		percentTraveled = 1.0 - (self.y - self.ceiling.height)*1.0 / (self.initialY - self.ceiling.height)
		
		linearX = self.initialX + (self.destX - self.initialX)*percentTraveled
		sineValue = math.sin(percentTraveled*self.period)
		wiggleAmount = (sineValue*20.0)
		self.x = linearX + wiggleAmount
		
		self.yOffset = self.yOffsetTarget * percentTraveled

		if self.y < self.ceiling.height:
			self.kill()
				
		
	def update( self ):
		self.rect.centerx = self.x
		self.rect.centery = self.y + self.yOffset + self.daisyChain.camera.getCameraHeightOffset()

		
		
