#------------------------------------------------------------------
# Contact: Jason Pratt
# Created: Jan 2009
#------------------------------------------------------------------

import time
import BaseHTTPServer
import urlparse
import json
import traceback
import os
import os.path
import sys

import util

if not os.path.exists( "newGameData" ):
	os.mkdir( "newGameData" )
if not os.path.exists( "usedGameData" ):
	os.mkdir( "usedGameData" )

class HandsomeHandler( BaseHTTPServer.BaseHTTPRequestHandler ):
	def do_HEAD( self ):
		self.send_response( 200 )
		self.send_header( "Content-type", "application/json" )
		self.end_headers()

	def do_GET( self ):
		print "do_GET"
		try:
			queryData = urlparse.parse_qs( urlparse.urlparse( self.path ).query )
			if "netVersion" in queryData:
				if queryData["netVersion"][0] == str( util.netVersion ):
					if "messageType" in queryData:
						if queryData["messageType"][0] == "getGameData":
							gameData = self.getNextGameData()
							dataString = json.dumps( {"responseType" : "getGameData", "gameData" : gameData} ) + "\n"
							self.send_response( 200 )
							self.send_header("Content-type", "application/json")
							self.end_headers()
							self.wfile.write( dataString )
						else:
							self.sendError( "Unrecognized messageType: %s" % queryData["messageType"][0] )
					else:
						self.sendError( "No messageType supplied" )
				else:
					self.sendError( "Bad version; expected: %d, got: %s" % (util.netVersion, queryData["netVersion"][0]) )
			else:
				self.sendError( "No netVersion supplied" )
		except:
			traceback.print_exc( file=sys.stdout )
			self.sendError( "Unexpected server error: %s" % traceback.format_exc() )

	def do_POST( self ):
		print "do_POST"
		try:
			queryData = urlparse.parse_qs( urlparse.urlparse( self.path ).query )
			if "netVersion" in queryData:
				if queryData["netVersion"][0] == str( util.netVersion ):
					if "messageType" in queryData:
						if queryData["messageType"][0] == "postGameData":
							dataString = self.rfile.readline()
							if len( dataString ) <= util.maxDataStringSize:
								outfile = open( os.path.join( "newGameData", "gameData_%d.json" % int( time.time()*1000.0 ) ), "w" )
								outfile.write( dataString )
								print "wrote file:", outfile
								outfile.close()

								self.send_response( 200 )
								self.send_header( "Content-type", "application/json" )
								self.end_headers()
								self.wfile.write( json.dumps( {"responseType" : "success"} ) + "\n" )
								print "sent response"
							else:
								self.sendError( "Data string too long: %d bytes" % len( dataString ) )
						else:
							self.sendError( "Unrecognized messageType: %s" % queryData["messageType"][0] )
					else:
						self.sendError( "No messageType supplied" )
				else:
					self.sendError( "Bad version; expected: %d, got: %s" % (util.netVersion, queryData["netVersion"][0]) )
			else:
				self.sendError( "No netVersion supplied" )
		except:
			traceback.print_exc( file=sys.stdout )
			self.sendError( "Unexpected server error: %s" % traceback.format_exc() )

	def sendError( self, errorMessage ):
		print "Sending error:", errorMessage
		data = {"responseType" : "Error",
		        "errorMessage" : errorMessage,}
		dataString = json.dumps( data )

		self.send_response( 400 )
		self.send_header( "Content-type", "application/json" )
		self.end_headers()
		self.wfile.write( dataString )

	def getNextGameData( self ):
		newGameDataFiles = os.listdir( "newGameData" )
		usedGameDataFiles = os.listdir( "usedGameData" )

		if len( newGameDataFiles ) > 0:
			newGameDataFiles.sort()
			filename = os.path.join( "newGameData", newGameDataFiles[0] )
			usedFilename = os.path.join( "usedGameData", newGameDataFiles[0] )
			gameDataString = open( filename ).read()
			try:
				os.rename( filename, usedFilename )
			except:
				traceback.print_exc( file=sys.stdout )
		elif len( usedGameDataFiles ) > 0:
			usedGameDataFiles.sort()
			filename = os.path.join( "usedGameData", usedGameDataFiles[0] )
			gameDataString = open( filename ).read()
		else:
			filename = os.path.join( os.path.dirname( __file__ ), "defaultGameData.json" )
			gameDataString = open( filename ).read()
		
		return json.loads( gameDataString )

	def log_message( self, format, *args ):
		print "log_message:",
		print format % args

if __name__ == "__main__":
	httpd = BaseHTTPServer.HTTPServer( (util.serverName, util.serverPort), HandsomeHandler )
	while True:
		try:
			httpd.serve_forever()
		except KeyboardInterrupt:
			break
		except:
			pass
	httpd.server_close()

